; *********************************************************
; ID reading/converting/checking routines
;
		RADIX dec

		list	p=12F629           ; list directive to define processor
        #include <p12F629.inc>
        errorlevel -302			; suppress "register not in bank0, check page bits" message.	
;

#define MAXIDs	24

		EXTERN	receive,rxreg

EE		CODE	0x2100
		DB	0					; NUMIDs = 0

id_vars	UDATA_SHR
aux		RES 1		; for calculations
charcnt	RES	1		; count chars to get
IDbuf	RES 5

; EEPROM addresses
NUMIDs	EQU 0			; # IDs stored
IDs		EQU	1			; ID storage address


id_code	CODE

getID	call receive					; Wait for STX
		movlw 2
		subwf rxreg,w					; STX ?
		btfss STATUS,Z
		goto getID						; No, loop
		movlw 5
		movwf charcnt					; Init 2* # ID chars to get
		movlw IDbuf						; Initialize pointer
		movwf FSR
rxloop	call receive					; Get next char
		call ASCII2bin					; to bin
		movwf INDF						; in buffer
		swapf INDF,f					; high nibble
		call receive					; Get next char
		call ASCII2bin					; to bin
		iorwf INDF,f					; in buffer (low nibble)
		incf FSR,f						; next ID byte
		decfsz charcnt,f				; loop
		goto rxloop
		call receive					; Wait for CR, discard
		call receive					; Wait for LF, discard
		call receive					; Wait for ETX
		movlw 3
		subwf rxreg,w					; ETX ?
		btfss STATUS,Z
		goto getID						; No, loop
		return

ASCII2bin
		movlw 041h
		subwf rxreg,w
		btfss STATUS,C
		addlw 7
		addlw 10
		return

addID	call numID					; Get NUMIDs in W
		movwf aux					; move to aux var
		sublw MAXIDs-1				; Full ?
		btfss STATUS,C
		return						; then ignore
		bsf STATUS,RP0 				; Bank 1
		incf EEDATA,f				; add 1 to NUMIDs
		bsf EECON1,WREN 			; Enable write
		call WriteEE				; do it
		bcf STATUS,C				; ID address is 5xNUMIDs
		movf aux,w
		rlf aux,f					; x2
		rlf aux,f					; x4
		addwf aux,w					; x5
		movwf EEADR
		incf EEADR,f				; offset by NUMIDs
		movlw 5
		movwf charcnt
		movlw IDbuf					; Initialize pointer
		movwf FSR
wIDloop	movf INDF,w					; get from buffer
		movwf EEDATA				; put to EEPROM
		call WriteEE				; do it
		incf FSR,f					; next ID byte
		incf EEADR,f				; next byte in storage
		decfsz charcnt,f			; loop
		goto wIDloop
		bcf EECON1,WREN 			; Disable write
		goto exit

numID	bsf STATUS,RP0 				; Bank 1
		clrf EEADR					; get NUMIDs
		bsf EECON1,RD 				; EE Read
		movf EEDATA,W 				; Move data to W
		goto exit

matchID	bsf STATUS,RP0 				; Bank 1
		clrf EEADR					; get NUMIDs
		bsf EECON1,RD 				; EE Read
		movf EEDATA,W 				; Move data to W
		movwf aux					; then to counter
		incf EEADR,f				; Point to ID storage
mIDl2	movlw 5						; Initialize ID lenght
		movwf charcnt
		movlw IDbuf					; Initialize pointer
		movwf FSR
mIDl1	bsf EECON1,RD 				; EE Read
		movf EEDATA,w				; Get byte from EEPROM
		subwf INDF,w				; compare with buffer
		btfss STATUS,Z				; match ?
		goto mIDnext				; go for next ID
		incf FSR,f					; next byte in buffer
		incf EEADR,f				; next byte in storage
		decfsz charcnt,f			; loop
		goto mIDl1
		movf aux,w					; MATCH! return ID# in W and Z reset
exit	bcf STATUS,RP0 				; Bank 0
		return
mIDnext	movf charcnt,w				; point to next ID
		addwf EEADR,f
		decfsz aux,f				; next ID
		goto mIDl2
		bsf	STATUS,Z
		goto exit					; No match, return Z set

purgeDB	bsf STATUS,RP0 				; Bank 1
		bsf EECON1,WREN 			; Enable write
		clrf EEADR					; point to NUMIDs
		movlw 1						; erase all but first ID
		movwf EEDATA
		call WriteEE				; do it
		bcf EECON1,WREN 			; Disable write
		goto exit

WriteEE	; ints are disabled
		movlw 055h 				; Unlock write
		movwf EECON2 
		movlw 0AAh 
		movwf EECON2 
		bsf EECON1,WR 			; Start the write
WEl1	btfsc EECON1,WR 		; Wait till done
		goto WEl1
		return


		GLOBAL	getID, addID, matchID, numID, purgeDB

		end



